#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Constants related to the APPROLE auth method."""

DEFAULT_MOUNT_POINT = 'approle'
ALLOWED_TOKEN_TYPES = ['service', 'batch', 'default', 'default-service', 'default-batch']
